@@ -43,9 +43,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include "busybox.h"
-#define bb_need_full_version
-#define BB_DECLARE_EXTERN
-#include "messages.c"
 #ifdef BB_SYSLOGD
 # include <sys/syslog.h>
 #endif
@@ -78,6 +75,12 @@ struct serial_struct {
 };
 
 
+#if (__GNU_LIBRARY__ > 5) || defined(__dietlibc__) 
+  #include <sys/reboot.h>
+  #define init_reboot(magic) reboot(magic)
+#else
+  #define init_reboot(magic) reboot(0xfee1dead, 672274793, magic)
+#endif
 
 #ifndef RB_HALT_SYSTEM
 static const int RB_HALT_SYSTEM = 0xcdef0123;
@@ -87,18 +90,10 @@ static const int RB_DISABLE_CAD = 0;
 static const int RB_AUTOBOOT = 0x01234567;
 #endif
 
-#if __GNU_LIBRARY__ > 5
-  #include <sys/reboot.h>
-  #define init_reboot(magic) reboot(magic)
-#else
-  #define init_reboot(magic) reboot(0xfee1dead, 672274793, magic)
-#endif
-
 #ifndef _PATH_STDPATH
 #define _PATH_STDPATH	"/usr/bin:/bin:/usr/sbin:/sbin"
 #endif
 
-
 #if defined BB_FEATURE_INIT_COREDUMPS
 /*
  * When a file named CORE_ENABLE_FLAG_FILE exists, setrlimit is called 
@@ -119,15 +114,8 @@ static const int RB_AUTOBOOT = 0x01234567;
 	extern int bdflush (int func, long int data);
 #endif
 
-
-#define VT_PRIMARY   "/dev/tty1"     /* Primary virtual console */
-#define VT_SECONDARY "/dev/tty2"     /* Virtual console */
-#define VT_THIRD     "/dev/tty3"     /* Virtual console */
-#define VT_FOURTH    "/dev/tty4"     /* Virtual console */
-#define VT_LOG       "/dev/tty5"     /* Virtual console */
-#define SERIAL_CON0  "/dev/ttyS0"    /* Primary serial console */
-#define SERIAL_CON1  "/dev/ttyS1"    /* Serial console */
-#define SHELL        "-/bin/sh"	     /* Default shell */
+#define SHELL        "/bin/sh"	     /* Default shell */
+#define LOGIN_SHELL  "-" SHELL	     /* Default login shell */
 #define INITTAB      "/etc/inittab"  /* inittab file location */
 #ifndef INIT_SCRIPT
 #define INIT_SCRIPT  "/etc/init.d/rcS"   /* Default sysinit script. */
@@ -137,6 +125,11 @@ static const int RB_AUTOBOOT = 0x01234567;
 //static const int MAXENV = 16;	/* Number of env. vars */
 static const int LOG = 0x1;
 static const int CONSOLE = 0x2;
+#if defined BB_FEATURE_EXTRA_QUIET
+static const int MAYBE_CONSOLE = 0x0;
+#else
+#define MAYBE_CONSOLE	CONSOLE
+#endif
 
 /* Allowed init action types */
 typedef enum {
@@ -178,20 +171,29 @@ struct initActionTag {
 static initAction *initActionList = NULL;
 
 
-static char *secondConsole = VT_SECONDARY;
-static char *thirdConsole  = VT_THIRD;
-static char *fourthConsole = VT_FOURTH;
-static char *log           = VT_LOG;
+static char *secondConsole = VC_2;
+static char *thirdConsole  = VC_3;
+static char *fourthConsole = VC_4;
+static char *log           = VC_5;
 static int  kernelVersion  = 0;
 static char termType[32]   = "TERM=linux";
 static char console[32]    = _PATH_CONSOLE;
 
 static void delete_initAction(initAction * action);
 
-
+static void loop_forever()
+{
+	while (1)
+		sleep (1);
+}
 
 /* Print a message to the specified device.
  * Device may be bitwise-or'd from LOG | CONSOLE */
+#ifdef DEBUG_INIT
+static inline messageND(int device, char *fmt, ...) { }
+#else 
+#define messageND message
+#endif
 static void message(int device, char *fmt, ...)
 		   __attribute__ ((format (printf, 2, 3)));
 static void message(int device, char *fmt, ...)
@@ -225,7 +227,7 @@ static void message(int device, char *fmt, ...)
 			device = CONSOLE;
 		} else if ((log_fd = device_open(log, O_RDWR|O_NDELAY)) < 0) {
 			log_fd = -2;
-			fprintf(stderr, "Bummer, can't write to log on %s!\r\n", log);
+			fprintf(stderr, "Bummer, can't write to log on %s!\n", log);
 			log = NULL;
 			device = CONSOLE;
 		}
@@ -278,7 +280,8 @@ static void set_term(int fd)
 
 	/* Make it be sane */
 	tty.c_cflag &= CBAUD|CBAUDEX|CSIZE|CSTOPB|PARENB|PARODD;
-	tty.c_cflag |= HUPCL|CLOCAL;
+	tty.c_cflag |= CREAD|HUPCL|CLOCAL;
+
 
 	/* input modes */
 	tty.c_iflag = ICRNL | IXON | IXOFF;
@@ -330,7 +333,7 @@ static void console_init()
 	}
 
 	if ((s = getenv("CONSOLE")) != NULL) {
-		snprintf(console, sizeof(console) - 1, "%s", s);
+		safe_strncpy(console, s, sizeof(console));
 	}
 #if #cpu(sparc)
 	/* sparc kernel supports console=tty[ab] parameter which is also 
@@ -338,22 +341,21 @@ static void console_init()
 	else if ((s = getenv("console")) != NULL) {
 		/* remap tty[ab] to /dev/ttyS[01] */
 		if (strcmp(s, "ttya") == 0)
-			snprintf(console, sizeof(console) - 1, "%s", SERIAL_CON0);
+			safe_strncpy(console, SC_0, sizeof(console));
 		else if (strcmp(s, "ttyb") == 0)
-			snprintf(console, sizeof(console) - 1, "%s", SERIAL_CON1);
+			safe_strncpy(console, SC_1, sizeof(console));
 	}
 #endif
 	else {
 		/* 2.2 kernels: identify the real console backend and try to use it */
 		if (ioctl(0, TIOCGSERIAL, &sr) == 0) {
 			/* this is a serial console */
-			snprintf(console, sizeof(console) - 1, "/dev/ttyS%d", sr.line);
+			snprintf(console, sizeof(console) - 1, SC_FORMAT, sr.line);
 		} else if (ioctl(0, VT_GETSTATE, &vt) == 0) {
 			/* this is linux virtual tty */
-			snprintf(console, sizeof(console) - 1, "/dev/tty%d",
-					 vt.v_active);
+			snprintf(console, sizeof(console) - 1, VC_FORMAT, vt.v_active);
 		} else {
-			snprintf(console, sizeof(console) - 1, "%s", _PATH_CONSOLE);
+			safe_strncpy(console, _PATH_CONSOLE, sizeof(console));
 			tried_devcons++;
 		}
 	}
@@ -362,20 +364,20 @@ static void console_init()
 		/* Can't open selected console -- try /dev/console */
 		if (!tried_devcons) {
 			tried_devcons++;
-			snprintf(console, sizeof(console) - 1, "%s", _PATH_CONSOLE);
+			safe_strncpy(console, _PATH_CONSOLE, sizeof(console));
 			continue;
 		}
 		/* Can't open selected console -- try vt1 */
 		if (!tried_vtprimary) {
 			tried_vtprimary++;
-			snprintf(console, sizeof(console) - 1, "%s", VT_PRIMARY);
+			safe_strncpy(console, VC_1, sizeof(console));
 			continue;
 		}
 		break;
 	}
 	if (fd < 0) {
 		/* Perhaps we should panic here? */
-		snprintf(console, sizeof(console) - 1, "/dev/null");
+		safe_strncpy(console, "/dev/null", sizeof(console));
 	} else {
 		/* check for serial console and disable logging to tty5 & running a
 		   * shell to tty2-4 */
@@ -387,9 +389,9 @@ static void console_init()
 			/* Force the TERM setting to vt102 for serial console --
 			 * iff TERM is set to linux (the default) */
 			if (strcmp( termType, "TERM=linux" ) == 0)
-				snprintf(termType, sizeof(termType) - 1, "TERM=vt102");
+				safe_strncpy(termType, "TERM=vt102", sizeof(termType));
 			message(LOG | CONSOLE,
-					"serial console detected.  Disabling virtual terminals.\r\n");
+					"serial console detected.  Disabling virtual terminals.\n");
 		}
 		close(fd);
 	}
@@ -402,7 +404,7 @@ static void fixup_argv(int argc, char **argv, char *new_argv0)
 	/* Fix up argv[0] to be certain we claim to be init */
 	len = strlen(argv[0]);
 	memset(argv[0], 0, len);
-	strncpy(argv[0], new_argv0, len);
+	safe_strncpy(argv[0], new_argv0, len + 1);
 
 	/* Wipe argv[1]-argv[N] so they don't clutter the ps listing */
 	len = 1;
@@ -432,8 +434,8 @@ static pid_t run(char *command, char *terminal, int get_enter)
 	char *environment[MAXENV+1] = {
 		termType,
 		"HOME=/",
-		"PATH=/usr/bin:/bin:/usr/sbin:/sbin",
-		"SHELL=/bin/sh",
+		"PATH=" _PATH_STDPATH,
+		"SHELL=" SHELL,
 		"USER=root",
 		NULL
 	};
@@ -451,7 +453,12 @@ static pid_t run(char *command, char *terminal, int get_enter)
 		}
 	}
 
-	if ((pid = fork()) == 0) {
+#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
+	if ((pid = fork()) == 0) 
+#else
+	if ((pid = vfork()) == 0) 
+#endif
+	{
 		/* Clean up */
 		ioctl(0, TIOCNOTTY, 0);
 		close(0);
@@ -472,7 +479,7 @@ static pid_t run(char *command, char *terminal, int get_enter)
 						terminal);
 				exit(1);
 			}
-			message(LOG | CONSOLE, "Bummer, can't open %s\r\n", terminal);
+			message(LOG | CONSOLE, "Bummer, can't open %s\n", terminal);
 			exit(1);
 		}
 		dup2(fd, 0);
@@ -487,13 +494,16 @@ static pid_t run(char *command, char *terminal, int get_enter)
 			cmd[0] = SHELL;
 			cmd[1] = "-c";
 			strcpy(buf, "exec ");
-			strncat(buf, command, sizeof(buf) - strlen(buf) - 1);
+			safe_strncpy(buf + sizeof("exec "), command, 
+					sizeof(buf) - sizeof("exec "));
 			cmd[2] = buf;
 			cmd[3] = NULL;
 		} else {
 			/* Convert command (char*) into cmd (char**, one word per string) */
-			for (tmpCmd = command, i = 0;
-					(tmpCmd = strsep(&command, " \t")) != NULL;) {
+			safe_strncpy(buf, command, sizeof(buf));
+			s = buf;
+			for (tmpCmd = buf, i = 0;
+					(tmpCmd = strsep(&s, " \t")) != NULL;) {
 				if (*tmpCmd != '\0') {
 					cmd[i] = tmpCmd;
 					tmpCmd++;
@@ -539,19 +549,15 @@ static pid_t run(char *command, char *terminal, int get_enter)
 			 * be allowed to start a shell or whatever an init script 
 			 * specifies.
 			 */
-#ifdef DEBUG_INIT
-			message(LOG, "Waiting for enter to start '%s' (pid %d, console %s)\r\n",
+			messageND(LOG, "Waiting for enter to start '%s' (pid %d, console %s)\n",
 					cmd[0], getpid(), terminal);
-#endif
 			write(fileno(stdout), press_enter, sizeof(press_enter) - 1);
 			getc(stdin);
 		}
 
-#ifdef DEBUG_INIT
 		/* Log the process name and args */
-		message(LOG, "Starting pid %d, console %s: '%s'\r\n",
+		messageND(LOG, "Starting pid %d, console %s: '%s'\n",
 				getpid(), terminal, command);
-#endif
 
 #if defined BB_FEATURE_INIT_COREDUMPS
 		if (stat (CORE_ENABLE_FLAG_FILE, &sb) == 0) {
@@ -599,6 +605,7 @@ static void check_memory()
 	if (check_free_memory() > 1000)
 		return;
 
+#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
 	if (stat("/etc/fstab", &statBuf) == 0) {
 		/* swapon -a requires /proc typically */
 		waitfor("mount proc /proc -t proc", console, FALSE);
@@ -609,12 +616,12 @@ static void check_memory()
 	} else
 		goto goodnight;
 	return;
+#endif
 
   goodnight:
 	message(CONSOLE,
-			"Sorry, your computer does not have enough memory.\r\n");
-	while (1)
-		sleep(1);
+			"Sorry, your computer does not have enough memory.\n");
+	loop_forever();
 }
 
 /* Run all commands to be run right before halt/reboot */
@@ -634,27 +641,35 @@ static void run_actions(initActionEnum action)
 #ifndef DEBUG_INIT
 static void shutdown_system(void)
 {
+	sigset_t block_signals;
 
-	/* first disable our SIGHUP signal */
-	signal(SIGHUP, SIG_DFL);
+	/* first disable all our signals */
+	sigemptyset(&block_signals);
+	sigaddset(&block_signals, SIGHUP);
+	sigaddset(&block_signals, SIGCHLD);
+	sigaddset(&block_signals, SIGUSR1);
+	sigaddset(&block_signals, SIGUSR2);
+	sigaddset(&block_signals, SIGINT);
+	sigaddset(&block_signals, SIGTERM);
+	sigprocmask(SIG_BLOCK, &block_signals, NULL);
 
 	/* Allow Ctrl-Alt-Del to reboot system. */
 	init_reboot(RB_ENABLE_CAD);
 
-	message(CONSOLE|LOG, "\r\nThe system is going down NOW !!\r\n");
+	message(CONSOLE|LOG, "\nThe system is going down NOW !!\n");
 	sync();
 
 	/* Send signals to every process _except_ pid 1 */
-	message(CONSOLE|LOG, "Sending SIGTERM to all processes.\r\n");
+	message(CONSOLE|LOG, "Sending SIGTERM to all processes.\n");
 	kill(-1, SIGTERM);
 	sleep(1);
 	sync();
 
-	message(CONSOLE|LOG, "Sending SIGKILL to all processes.\r\n");
+	message(CONSOLE|LOG, "Sending SIGKILL to all processes.\n");
 	kill(-1, SIGKILL);
 	sleep(1);
 
-	/* run everything to be run at "ctrlaltdel" */
+	/* run everything to be run at "shutdown" */
 	run_actions(SHUTDOWN);
 
 	sync();
@@ -669,9 +684,15 @@ static void halt_signal(int sig)
 {
 	shutdown_system();
 	message(CONSOLE|LOG,
-			"The system is halted. Press %s or turn off power\r\n",
-			(secondConsole == NULL)	/* serial console */
-			? "Reset" : "CTRL-ALT-DEL");
+#if #cpu(s390)
+			/* Seems the s390 console is Wierd(tm). */
+			"The system is halted. You may reboot now.\n",
+#else
+			/* secondConsole is NULL for a serial console */
+			"The system is halted. Press %s or turn off power\n",
+			(secondConsole == NULL)? "Reset" : "CTRL-ALT-DEL"
+#endif
+		   );
 	sync();
 
 	/* allow time for last message to reach serial console */
@@ -681,20 +702,22 @@ static void halt_signal(int sig)
 		init_reboot(RB_POWER_OFF);
 	else
 		init_reboot(RB_HALT_SYSTEM);
-	exit(0);
+
+	loop_forever();
 }
 
 static void reboot_signal(int sig)
 {
 	shutdown_system();
-	message(CONSOLE|LOG, "Please stand by while rebooting the system.\r\n");
+	message(CONSOLE|LOG, "Please stand by while rebooting the system.\n");
 	sync();
 
 	/* allow time for last message to reach serial console */
 	sleep(2);
 
 	init_reboot(RB_AUTOBOOT);
-	exit(0);
+
+	loop_forever();
 }
 
 static void ctrlaltdel_signal(int sig)
@@ -718,18 +741,20 @@ static void new_initAction(initActionEnum action, char *process, char *cons)
 	if (secondConsole == NULL && strcmp(cons, console)
 		&& strcmp(cons, "/dev/null"))
 		return;
+	if (strcmp(cons, "/dev/null") == 0 && action == ASKFIRST)
+		return;
+
 
 	newAction = calloc((size_t) (1), sizeof(initAction));
 	if (!newAction) {
 		message(LOG | CONSOLE, "Memory allocation failure\n");
-		while (1)
-			sleep(1);
+		loop_forever();
 	}
 	newAction->nextPtr = initActionList;
 	initActionList = newAction;
-	strncpy(newAction->process, process, 255);
+	safe_strncpy(newAction->process, process, 255);
 	newAction->action = action;
-	strncpy(newAction->console, cons, 255);
+	safe_strncpy(newAction->console, cons, 255);
 	newAction->pid = 0;
 //    message(LOG|CONSOLE, "process='%s' action='%d' console='%s'\n",
 //      newAction->process, newAction->action, newAction->console);
@@ -754,7 +779,7 @@ static void delete_initAction(initAction * action)
 
 /* NOTE that if BB_FEATURE_USE_INITTAB is NOT defined,
  * then parse_inittab() simply adds in some default
- * actions(i.e runs INIT_SCRIPT and then starts a pair 
+ * actions(i.e., runs INIT_SCRIPT and then starts a pair 
  * of "askfirst" shells).  If BB_FEATURE_USE_INITTAB 
  * _is_ defined, but /etc/inittab is missing, this 
  * results in the same set of default behaviors.
@@ -775,21 +800,23 @@ static void parse_inittab(void)
 #endif
 		/* Reboot on Ctrl-Alt-Del */
 		new_initAction(CTRLALTDEL, "/sbin/reboot", console);
+#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
 		/* Swapoff on halt/reboot */
 		new_initAction(SHUTDOWN, "/sbin/swapoff -a", console);
+#endif
 		/* Umount all filesystems on halt/reboot */
 		new_initAction(SHUTDOWN, "/bin/umount -a -r", console);
 		/* Askfirst shell on tty1 */
-		new_initAction(ASKFIRST, SHELL, console);
+		new_initAction(ASKFIRST, LOGIN_SHELL, console);
 		/* Askfirst shell on tty2 */
 		if (secondConsole != NULL)
-			new_initAction(ASKFIRST, SHELL, secondConsole);
+			new_initAction(ASKFIRST, LOGIN_SHELL, secondConsole);
 		/* Askfirst shell on tty3 */
 		if (thirdConsole != NULL)
-			new_initAction(ASKFIRST, SHELL, thirdConsole);
+			new_initAction(ASKFIRST, LOGIN_SHELL, thirdConsole);
 		/* Askfirst shell on tty4 */
 		if (fourthConsole != NULL)
-			new_initAction(ASKFIRST, SHELL, fourthConsole);
+			new_initAction(ASKFIRST, LOGIN_SHELL, fourthConsole);
 		/* sysinit */
 		new_initAction(SYSINIT, INIT_SCRIPT, console);
 
@@ -902,13 +929,14 @@ extern int init_main(int argc, char **argv)
 	/* Figure out what kernel this is running */
 	kernelVersion = get_kernel_revision();
 
-	/* Figure out where the default console should be */
-	console_init();
-
 	/* Close whatever files are open, and reset the console. */
 	close(0);
 	close(1);
 	close(2);
+
+	/* Figure out where the default console should be */
+	console_init();
+
 	set_term(0);
 	chdir("/");
 	setsid();
@@ -917,22 +945,7 @@ extern int init_main(int argc, char **argv)
 	putenv("PATH="_PATH_STDPATH);
 
 	/* Hello world */
-#ifndef DEBUG_INIT
-	message(
-#if ! defined BB_FEATURE_EXTRA_QUIET
-			CONSOLE|
-#endif
-			LOG,
-			"init started:  %s\r\n", full_version);
-#else
-	message(
-#if ! defined BB_FEATURE_EXTRA_QUIET
-			CONSOLE|
-#endif
-			LOG,
-			"init(%d) started:  %s\r\n", getpid(), full_version);
-#endif
-
+	message(MAYBE_CONSOLE|LOG, "init started:  %s\n", full_version);
 
 	/* Make sure there is enough memory to do something useful. */
 	check_memory();
@@ -942,19 +955,19 @@ extern int init_main(int argc, char **argv)
 					 !strcmp(argv[1], "-s") || !strcmp(argv[1], "1"))) {
 		/* Ask first then start a shell on tty2-4 */
 		if (secondConsole != NULL)
-			new_initAction(ASKFIRST, SHELL, secondConsole);
+			new_initAction(ASKFIRST, LOGIN_SHELL, secondConsole);
 		if (thirdConsole != NULL)
-			new_initAction(ASKFIRST, SHELL, thirdConsole);
+			new_initAction(ASKFIRST, LOGIN_SHELL, thirdConsole);
 		if (fourthConsole != NULL)
-			new_initAction(ASKFIRST, SHELL, fourthConsole);
+			new_initAction(ASKFIRST, LOGIN_SHELL, fourthConsole);
 		/* Start a shell on tty1 */
-		new_initAction(RESPAWN, SHELL, console);
+		new_initAction(RESPAWN, LOGIN_SHELL, console);
 	} else {
 		/* Not in single user mode -- see what inittab says */
 
 		/* NOTE that if BB_FEATURE_USE_INITTAB is NOT defined,
 		 * then parse_inittab() simply adds in some default
-		 * actions(i.e runs INIT_SCRIPT and then starts a pair 
+		 * actions(i.e., runs INIT_SCRIPT and then starts a pair 
 		 * of "askfirst" shells */
 		parse_inittab();
 	}
@@ -965,23 +978,11 @@ extern int init_main(int argc, char **argv)
 	/* Now run everything that needs to be run */
 
 	/* First run the sysinit command */
-	for (a = initActionList; a; a = tmp) {
-		tmp = a->nextPtr;
-		if (a->action == SYSINIT) {
-			waitfor(a->process, a->console, FALSE);
-			/* Now remove the "sysinit" entry from the list */
-			delete_initAction(a);
-		}
-	}
+	run_actions(SYSINIT);
+
 	/* Next run anything that wants to block */
-	for (a = initActionList; a; a = tmp) {
-		tmp = a->nextPtr;
-		if (a->action == WAIT) {
-			waitfor(a->process, a->console, FALSE);
-			/* Now remove the "wait" entry from the list */
-			delete_initAction(a);
-		}
-	}
+	run_actions(WAIT);
+
 	/* Next run anything to be run only once */
 	for (a = initActionList; a; a = tmp) {
 		tmp = a->nextPtr;
@@ -995,8 +996,7 @@ extern int init_main(int argc, char **argv)
 	if (initActionList == NULL) {
 		message(LOG | CONSOLE,
 				"No more tasks for init -- sleeping forever.\n");
-		while (1)
-			sleep(1);
+		loop_forever();
 	}
 
 	/* Now run the looping stuff for the rest of forever */
