@@ -20,10 +20,35 @@
 /* Written by Ulrich Drepper <drepper@gnu.ai.mit.edu> */
 /* Hacked to work with BusyBox by Alfred M. Szmidt <ams@trillian.itslinux.org> */
 
+/*
+ * June 29, 2001        Manuel Novoa III
+ *
+ * Added MD5SUM_SIZE_VS_SPEED configuration option.
+ *
+ * Current valid values, with data from my system for comparison, are:
+ *   (using uClibc and running on linux-2.4.4.tar.bz2)
+ *                     user times (sec)  text size (386)
+ *     0 (fastest)         1.1                6144
+ *     1                   1.4                5392
+ *     2                   3.0                5088
+ *     3 (smallest)        5.1                4912
+ */
+
+#define MD5SUM_SIZE_VS_SPEED 2
+
+/**********************************************************************/
+
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
 #include <getopt.h>
+#include <stdlib.h>
+#include <string.h>
+#include <endian.h>
+#include <sys/types.h>
+#if defined HAVE_LIMITS_H
+# include <limits.h>
+#endif
 #include "busybox.h"
 
 /* For some silly reason, this file uses backwards TRUE and FALSE conventions */
@@ -38,75 +63,17 @@
 
 /* md5.c - Functions to compute MD5 message digest of files or memory blocks
  *         according to the definition of MD5 in RFC 1321 from April 1992.
- * Copyright (C) 1995, 1996 Free Software Foundation, Inc.
- *
- * NOTE: The canonical source of this file is maintained with the GNU C
- * Library.  Bugs can be reported to bug-glibc@prep.ai.mit.edu.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 /* Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, 1995.  */
 
-#include <sys/types.h>
-#include <stdlib.h>
-#include <string.h>
-#include <endian.h>
-
-#include "busybox.h"
 //----------------------------------------------------------------------------
 //--------md5.h
 //----------------------------------------------------------------------------
 
 /* md5.h - Declaration of functions and data types used for MD5 sum
-   computing library functions.
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
-   NOTE: The canonical source of this file is maintained with the GNU C
-   Library.  Bugs can be reported to bug-glibc@prep.ai.mit.edu.
+   computing library functions. */
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; either version 2, or (at your option) any
-   later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
-
-#ifndef _MD5_H
-static const int _MD5_H = 1;
-
-#include <stdio.h>
-
-#if defined HAVE_LIMITS_H || defined _LIBC
-# include <limits.h>
-#endif
-
-/* The following contortions are an attempt to use the C preprocessor
-   to determine an unsigned integral type that is 32 bits wide.  An
-   alternative approach is to use autoconf's AC_CHECK_SIZEOF macro, but
-   doing that would require that the configure script compile and *run*
-   the resulting executable.  Locally running cross-compiled executables
-   is usually not possible.  */
-
-# include <sys/types.h>
 typedef u_int32_t md5_uint32;
 
 /* Structure to save state of computation between the single steps.  */
@@ -129,20 +96,20 @@ struct md5_ctx
 
 /* Initialize structure containing state of computation.
    (RFC 1321, 3.3: Step 3)  */
-extern void md5_init_ctx __P ((struct md5_ctx *ctx));
+static void md5_init_ctx __P ((struct md5_ctx *ctx));
 
 /* Starting with the result of former calls of this function (or the
    initialization function update the context for the next LEN bytes
    starting at BUFFER.
    It is necessary that LEN is a multiple of 64!!! */
-extern void md5_process_block __P ((const void *buffer, size_t len,
+static void md5_process_block __P ((const void *buffer, size_t len,
 				    struct md5_ctx *ctx));
 
 /* Starting with the result of former calls of this function (or the
    initialization function update the context for the next LEN bytes
    starting at BUFFER.
    It is NOT required that LEN is a multiple of 64.  */
-extern void md5_process_bytes __P ((const void *buffer, size_t len,
+static void md5_process_bytes __P ((const void *buffer, size_t len,
 				    struct md5_ctx *ctx));
 
 /* Process the remaining bytes in the buffer and put result from CTX
@@ -152,40 +119,40 @@ extern void md5_process_bytes __P ((const void *buffer, size_t len,
 
    IMPORTANT: On some systems it is required that RESBUF is correctly
    aligned for a 32 bits value.  */
-extern void *md5_finish_ctx __P ((struct md5_ctx *ctx, void *resbuf));
+static void *md5_finish_ctx __P ((struct md5_ctx *ctx, void *resbuf));
 
 
-/* Put result from CTX in first 16 bytes following RESBUF.  The result is
-   always in little endian byte order, so that a byte-wise output yields
-   to the wanted ASCII representation of the message digest.
-
-   IMPORTANT: On some systems it is required that RESBUF is correctly
-   aligned for a 32 bits value.  */
-extern void *md5_read_ctx __P ((const struct md5_ctx *ctx, void *resbuf));
 
 
 /* Compute MD5 message digest for bytes read from STREAM.  The
    resulting message digest number will be written into the 16 bytes
    beginning at RESBLOCK.  */
-extern int md5_stream __P ((FILE *stream, void *resblock));
+static int md5_stream __P ((FILE *stream, void *resblock));
 
 /* Compute MD5 message digest for LEN bytes beginning at BUFFER.  The
    result is always in little endian byte order, so that a byte-wise
    output yields to the wanted ASCII representation of the message
    digest.  */
-extern void *md5_buffer __P ((const char *buffer, size_t len, void *resblock));
-
-#endif
+static void *md5_buffer __P ((const char *buffer, size_t len, void *resblock));
 
 //----------------------------------------------------------------------------
 //--------end of md5.h
 //----------------------------------------------------------------------------
 
-#define SWAP(n) (n)
+/* Handle endian-ness */
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+	#define SWAP(n) (n)
+#else
+	#define SWAP(n) ((n << 24) | ((n&65280)<<8) | ((n&16711680)>>8) | (n>>24))
+#endif
 
+
+
+#if MD5SUM_SIZE_VS_SPEED == 0
 /* This array contains the bytes used to pad the buffer to the next
    64-byte boundary.  (RFC 1321, 3.1: Step 1)  */
 static const unsigned char fillbuf[64] = { 0x80, 0 /* , 0, 0, ...  */  };
+#endif
 
 /* Initialize structure containing state of computation.
    (RFC 1321, 3.3: Step 3)  */
@@ -200,27 +167,12 @@ void md5_init_ctx(struct md5_ctx *ctx)
   ctx->buflen = 0;
 }
 
-/* Put result from CTX in first 16 bytes following RESBUF.  The result
-   must be in little endian byte order.
-
-   IMPORTANT: On some systems it is required that RESBUF is correctly
-   aligned for a 32 bits value.  */
-void *md5_read_ctx(const struct md5_ctx *ctx, void *resbuf)
-{
-  ((md5_uint32 *) resbuf)[0] = SWAP(ctx->A);
-  ((md5_uint32 *) resbuf)[1] = SWAP(ctx->B);
-  ((md5_uint32 *) resbuf)[2] = SWAP(ctx->C);
-  ((md5_uint32 *) resbuf)[3] = SWAP(ctx->D);
-
-  return resbuf;
-}
-
 /* Process the remaining bytes in the internal buffer and the usual
    prolog according to the standard and write the result to RESBUF.
 
    IMPORTANT: On some systems it is required that RESBUF is correctly
    aligned for a 32 bits value.  */
-void *md5_finish_ctx(struct md5_ctx *ctx, void *resbuf)
+static void *md5_finish_ctx(struct md5_ctx *ctx, void *resbuf)
 {
   /* Take yet unprocessed bytes into account.  */
   md5_uint32 bytes = ctx->buflen;
@@ -232,23 +184,39 @@ void *md5_finish_ctx(struct md5_ctx *ctx, void *resbuf)
     ++ctx->total[1];
 
   pad = bytes >= 56 ? 64 + 56 - bytes : 56 - bytes;
+#if MD5SUM_SIZE_VS_SPEED > 0
+  memset(&ctx->buffer[bytes], 0, pad);
+  ctx->buffer[bytes] = 0x80;
+#else
   memcpy(&ctx->buffer[bytes], fillbuf, pad);
+#endif
 
   /* Put the 64-bit file length in *bits* at the end of the buffer.  */
   *(md5_uint32 *) & ctx->buffer[bytes + pad] = SWAP(ctx->total[0] << 3);
   *(md5_uint32 *) & ctx->buffer[bytes + pad + 4] =
-    SWAP((ctx->total[1] << 3) | (ctx->total[0] >> 29));
+    SWAP( ((ctx->total[1] << 3) | (ctx->total[0] >> 29)) );
 
   /* Process last bytes.  */
   md5_process_block(ctx->buffer, bytes + pad + 8, ctx);
 
-  return md5_read_ctx(ctx, resbuf);
+/* Put result from CTX in first 16 bytes following RESBUF.  The result is
+   always in little endian byte order, so that a byte-wise output yields
+   to the wanted ASCII representation of the message digest.
+
+   IMPORTANT: On some systems it is required that RESBUF is correctly
+   aligned for a 32 bits value.  */
+  ((md5_uint32 *) resbuf)[0] = SWAP(ctx->A);
+  ((md5_uint32 *) resbuf)[1] = SWAP(ctx->B);
+  ((md5_uint32 *) resbuf)[2] = SWAP(ctx->C);
+  ((md5_uint32 *) resbuf)[3] = SWAP(ctx->D);
+
+  return resbuf;
 }
 
 /* Compute MD5 message digest for bytes read from STREAM.  The
    resulting message digest number will be written into the 16 bytes
    beginning at RESBLOCK.  */
-int md5_stream(FILE *stream, void *resblock)
+static int md5_stream(FILE *stream, void *resblock)
 {
   /* Important: BLOCKSIZE must be a multiple of 64.  */
 static const int BLOCKSIZE = 4096;
@@ -300,7 +268,7 @@ static const int BLOCKSIZE = 4096;
    result is always in little endian byte order, so that a byte-wise
    output yields to the wanted ASCII representation of the message
    digest.  */
-void *md5_buffer(const char *buffer, size_t len, void *resblock)
+static void *md5_buffer(const char *buffer, size_t len, void *resblock)
 {
   struct md5_ctx ctx;
 
@@ -314,7 +282,7 @@ void *md5_buffer(const char *buffer, size_t len, void *resblock)
   return md5_finish_ctx(&ctx, resblock);
 }
 
-void md5_process_bytes(const void *buffer, size_t len, struct md5_ctx *ctx)
+static void md5_process_bytes(const void *buffer, size_t len, struct md5_ctx *ctx)
 {
   /* When we already have some bits in our internal buffer concatenate
      both inputs first.  */
@@ -362,12 +330,55 @@ void md5_process_bytes(const void *buffer, size_t len, struct md5_ctx *ctx)
 
 /* Process LEN bytes of BUFFER, accumulating context into CTX.
    It is assumed that LEN % 64 == 0.  */
-void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
+static void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
 {
   md5_uint32 correct_words[16];
   const md5_uint32 *words = buffer;
   size_t nwords = len / sizeof(md5_uint32);
   const md5_uint32 *endp = words + nwords;
+#if MD5SUM_SIZE_VS_SPEED > 0
+  static const md5_uint32 C_array[] = {
+      /* round 1 */
+      0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee,
+      0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501,
+      0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be,
+      0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821,
+      /* round 2 */
+      0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa,
+      0xd62f105d, 0x2441453,  0xd8a1e681, 0xe7d3fbc8,
+      0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed,
+      0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a,
+      /* round 3 */
+      0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c,
+      0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70,
+      0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x4881d05,
+      0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665,
+      /* round 4 */
+      0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039,
+      0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1,
+      0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1,
+      0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391
+  };
+
+  static const char P_array[] = {
+#if MD5SUM_SIZE_VS_SPEED > 1
+      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, /* 1 */
+#endif
+      1, 6, 11, 0, 5, 10, 15, 4, 9, 14, 3, 8, 13, 2, 7, 12, /* 2 */
+      5, 8, 11, 14, 1, 4, 7, 10, 13, 0, 3, 6, 9, 12, 15, 2, /* 3 */
+      0, 7, 14, 5, 12, 3, 10, 1, 8, 15, 6, 13, 4, 11, 2, 9  /* 4 */
+  };
+
+#if MD5SUM_SIZE_VS_SPEED > 1
+  static const char S_array[] = {
+      7, 12, 17, 22,
+      5, 9, 14, 20,
+      4, 11, 16, 23,
+      6, 10, 15, 21
+  };
+#endif
+#endif
+
   md5_uint32 A = ctx->A;
   md5_uint32 B = ctx->B;
   md5_uint32 C = ctx->C;
@@ -389,6 +400,78 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
     md5_uint32 C_save = C;
     md5_uint32 D_save = D;
 
+#if MD5SUM_SIZE_VS_SPEED > 1
+#define CYCLIC(w, s) (w = (w << s) | (w >> (32 - s)))
+
+    const md5_uint32 *pc;
+    const char *pp;
+    const char *ps;
+    int i;
+    md5_uint32 temp;
+
+    for ( i=0 ; i < 16 ; i++ ) {
+	cwp[i] = SWAP(words[i]);
+    }
+    words += 16;
+
+#if MD5SUM_SIZE_VS_SPEED > 2
+    pc = C_array; pp = P_array; ps = S_array - 4;
+
+    for ( i = 0 ; i < 64 ; i++ ) {
+	if ((i&0x0f) == 0) ps += 4;
+	temp = A;
+	switch (i>>4) {
+	    case 0:
+		temp += FF(B,C,D);
+		break;
+	    case 1:
+		temp += FG(B,C,D);
+		break;
+	    case 2:
+		temp += FH(B,C,D);
+		break;
+	    case 3:
+		temp += FI(B,C,D);
+	}
+	temp += cwp[(int)(*pp++)] + *pc++;
+	temp = CYCLIC (temp, ps[i&3]);
+	temp += B;
+	A = D; D = C; C = B; B = temp;
+    }
+#else
+    pc = C_array; pp = P_array; ps = S_array;
+
+    for ( i = 0 ; i < 16 ; i++ ) {
+	temp = A + FF(B,C,D) + cwp[(int)(*pp++)] + *pc++;
+	temp = CYCLIC (temp, ps[i&3]);
+	temp += B;
+	A = D; D = C; C = B; B = temp;
+    }
+
+    ps += 4;
+    for ( i = 0 ; i < 16 ; i++ ) {
+	temp = A + FG(B,C,D) + cwp[(int)(*pp++)] + *pc++;
+	temp = CYCLIC (temp, ps[i&3]);
+	temp += B;
+	A = D; D = C; C = B; B = temp;
+    }
+    ps += 4;
+    for ( i = 0 ; i < 16 ; i++ ) {
+	temp = A + FH(B,C,D) + cwp[(int)(*pp++)] + *pc++;
+	temp = CYCLIC (temp, ps[i&3]);
+	temp += B;
+	A = D; D = C; C = B; B = temp;
+    }
+    ps += 4;
+    for ( i = 0 ; i < 16 ; i++ ) {
+	temp = A + FI(B,C,D) + cwp[(int)(*pp++)] + *pc++;
+	temp = CYCLIC (temp, ps[i&3]);
+	temp += B;
+	A = D; D = C; C = B; B = temp;
+    }
+
+#endif
+#else
     /* First round: using the given function, the context and a constant
        the next context is computed.  Because the algorithms processing
        unit is a 32-bit word and it is determined to work on words in
@@ -408,6 +491,7 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
 
     /* It is unfortunate that C does not provide an operator for
        cyclic rotation.  Hope the C compiler is smart enough.  */
+    /* gcc 2.95.4 seems to be --aaronl */
 #define CYCLIC(w, s) (w = (w << s) | (w >> (32 - s)))
 
     /* Before we start, one word to the strange constants.
@@ -416,7 +500,22 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
        T[i] = (int) (4294967296.0 * fabs (sin (i))), i=1..64
     */
 
+#if MD5SUM_SIZE_VS_SPEED == 1
+    const md5_uint32 *pc;
+    const char *pp;
+    int i;
+#endif
+
     /* Round 1.  */
+#if MD5SUM_SIZE_VS_SPEED == 1
+    pc = C_array;
+    for ( i=0 ; i < 4 ; i++ ) {
+	OP(A, B, C, D, 7, *pc++);
+	OP(D, A, B, C, 12, *pc++);
+	OP(C, D, A, B, 17, *pc++);
+	OP(B, C, D, A, 22, *pc++);
+    }
+#else
     OP(A, B, C, D, 7, 0xd76aa478);
     OP(D, A, B, C, 12, 0xe8c7b756);
     OP(C, D, A, B, 17, 0x242070db);
@@ -433,6 +532,7 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
     OP(D, A, B, C, 12, 0xfd987193);
     OP(C, D, A, B, 17, 0xa679438e);
     OP(B, C, D, A, 22, 0x49b40821);
+#endif
 
     /* For the second to fourth round we have the possibly swapped words
        in CORRECT_WORDS.  Redefine the macro to take an additional first
@@ -448,6 +548,15 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
       while (0)
 
     /* Round 2.  */
+#if MD5SUM_SIZE_VS_SPEED == 1
+    pp = P_array;
+    for ( i=0 ; i < 4 ; i++ ) {
+	OP(FG, A, B, C, D, (int)(*pp++), 5, *pc++);
+	OP(FG, D, A, B, C, (int)(*pp++), 9, *pc++);
+	OP(FG, C, D, A, B, (int)(*pp++), 14, *pc++);
+	OP(FG, B, C, D, A, (int)(*pp++), 20, *pc++);
+    }
+#else
     OP(FG, A, B, C, D, 1, 5, 0xf61e2562);
     OP(FG, D, A, B, C, 6, 9, 0xc040b340);
     OP(FG, C, D, A, B, 11, 14, 0x265e5a51);
@@ -464,8 +573,17 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
     OP(FG, D, A, B, C, 2, 9, 0xfcefa3f8);
     OP(FG, C, D, A, B, 7, 14, 0x676f02d9);
     OP(FG, B, C, D, A, 12, 20, 0x8d2a4c8a);
+#endif
 
     /* Round 3.  */
+#if MD5SUM_SIZE_VS_SPEED == 1
+    for ( i=0 ; i < 4 ; i++ ) {
+	OP(FH, A, B, C, D, (int)(*pp++), 4, *pc++);
+	OP(FH, D, A, B, C, (int)(*pp++), 11, *pc++);
+	OP(FH, C, D, A, B, (int)(*pp++), 16, *pc++);
+	OP(FH, B, C, D, A, (int)(*pp++), 23, *pc++);
+    }
+#else
     OP(FH, A, B, C, D, 5, 4, 0xfffa3942);
     OP(FH, D, A, B, C, 8, 11, 0x8771f681);
     OP(FH, C, D, A, B, 11, 16, 0x6d9d6122);
@@ -482,8 +600,17 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
     OP(FH, D, A, B, C, 12, 11, 0xe6db99e5);
     OP(FH, C, D, A, B, 15, 16, 0x1fa27cf8);
     OP(FH, B, C, D, A, 2, 23, 0xc4ac5665);
+#endif
 
     /* Round 4.  */
+#if MD5SUM_SIZE_VS_SPEED == 1
+    for ( i=0 ; i < 4 ; i++ ) {
+	OP(FI, A, B, C, D, (int)(*pp++), 6, *pc++);
+	OP(FI, D, A, B, C, (int)(*pp++), 10, *pc++);
+	OP(FI, C, D, A, B, (int)(*pp++), 15, *pc++);
+	OP(FI, B, C, D, A, (int)(*pp++), 21, *pc++);
+    }
+#else
     OP(FI, A, B, C, D, 0, 6, 0xf4292244);
     OP(FI, D, A, B, C, 7, 10, 0x432aff97);
     OP(FI, C, D, A, B, 14, 15, 0xab9423a7);
@@ -500,6 +627,8 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
     OP(FI, D, A, B, C, 11, 10, 0xbd3af235);
     OP(FI, C, D, A, B, 2, 15, 0x2ad7d2bb);
     OP(FI, B, C, D, A, 9, 21, 0xeb86d391);
+#endif
+#endif
 
     /* Add the starting values of the context.  */
     A += A_save;
@@ -520,11 +649,7 @@ void md5_process_block(const void *buffer, size_t len, struct md5_ctx *ctx)
 //----------------------------------------------------------------------------
 
 #define ISWHITE(c) ((c) == ' ' || (c) == '\t')
-#define IN_CTYPE_DOMAIN(c) 1
-#define ISXDIGIT(c) (IN_CTYPE_DOMAIN (c) && isxdigit (c))
-#define STREQ(a, b) (strcmp ((a), (b)) == 0)
-#define TOLOWER(Ch) tolower (Ch)
-#define OPENOPTS(BINARY) "r"
+#define ISXDIGIT(c) (isxdigit (c))
 
 /* The minimum length of a valid digest line in a file produced
    by `md5sum FILE' and read by `md5sum -c'.  This length does
@@ -544,7 +669,6 @@ static int warn = 0; /* With -w, print a message to standard error warning
 static int split_3(char *s,
                    size_t s_len,
                    unsigned char **u,
-                   int *binary,
                    char **w)
 {
   size_t i = 0;
@@ -577,11 +701,10 @@ static int split_3(char *s,
 
   if (s[i] != ' ' && s[i] != '*')
     return FALSE;
-  *binary = (s[i++] == '*');
 
   /* All characters between the type indicator and end of line are
      significant -- that includes leading and trailing white space.  */
-  *w = &s[i];
+  *w = &s[++i];
 
   if (escaped_filename) {
     /* Translate each `\n' string in the file name to a NEWLINE,
@@ -625,7 +748,7 @@ static int split_3(char *s,
   return TRUE;
 }
 
-static int hex_digits(unsigned char const *s)
+static inline int hex_digits(unsigned char const *s)
 {
   while (*s) {
     if (!ISXDIGIT(*s))
@@ -639,21 +762,18 @@ static int hex_digits(unsigned char const *s)
    put the result in *MD5_RESULT.  Return non-zero upon failure, zero
    to indicate success.  */
 static int md5_file(const char *filename,
-                    int binary,
                     unsigned char *md5_result)
 {
   FILE *fp;
 
-  if (STREQ(filename, "-")) {
+  if (filename[0] == '-' && filename[1] == '\0') {
     have_read_stdin = 1;
     fp = stdin;
   } else {
-    fp = fopen(filename, OPENOPTS(binary));
-    if (fp == NULL) {
-      perror_msg("%s", filename);
+    fp = wfopen(filename, "r");
+    if (fp == NULL)
       return FALSE;
     }
-  }
 
   if (md5_stream(fp, md5_result)) {
     perror_msg("%s", filename);
@@ -681,22 +801,19 @@ static int md5_check(const char *checkfile_name)
   size_t line_number;
   char line[BUFSIZ];
 
-  if (STREQ(checkfile_name, "-")) {
+  if (checkfile_name[0] == '-' && checkfile_name[1] == '\0') {
     have_read_stdin = 1;
     checkfile_stream = stdin;
   } else {
-    checkfile_stream = fopen(checkfile_name, "r");
-    if (checkfile_stream == NULL) {
-      perror_msg("%s", checkfile_name);
+    checkfile_stream = wfopen(checkfile_name, "r");
+    if (checkfile_stream == NULL)
       return FALSE;
     }
-  }
 
   line_number = 0;
 
   do {
     char *filename;
-    int binary;
     unsigned char *md5num;
     int line_length;
 
@@ -716,7 +833,7 @@ static int md5_check(const char *checkfile_name)
     if (line[line_length - 1] == '\n')
       line[--line_length] = '\0';
 
-    if (split_3(line, line_length, &md5num, &binary, &filename)
+    if (split_3(line, line_length, &md5num, &filename)
         || !hex_digits(md5num)) {
       if (warn) {
         error_msg("%s: %lu: improperly formatted MD5 checksum line",
@@ -732,7 +849,7 @@ static int md5_check(const char *checkfile_name)
 
       ++n_properly_formated_lines;
 
-      if (md5_file(filename, binary, md5buffer)) {
+      if (md5_file(filename, md5buffer)) {
         ++n_open_or_read_failures;
         if (!status_only) {
           printf("%s: FAILED open or read\n", filename);
@@ -743,9 +860,9 @@ static int md5_check(const char *checkfile_name)
         /* Compare generated binary number with text representation
            in check file.  Ignore case of hex digits.  */
         for (cnt = 0; cnt < 16; ++cnt) {
-          if (TOLOWER(md5num[2 * cnt])
+          if (tolower(md5num[2 * cnt])
               != bin2hex[md5buffer[cnt] >> 4]
-              || (TOLOWER(md5num[2 * cnt + 1])
+              || (tolower(md5num[2 * cnt + 1])
                   != (bin2hex[md5buffer[cnt] & 0xf])))
             break;
         }
@@ -810,8 +927,8 @@ int md5sum_main(int argc,
   char **string = NULL;
   size_t n_strings = 0;
   size_t err = 0;
-  int file_type_specified = 0;
-  int binary = 0;
+  char file_type_specified = 0;
+  char binary = 0;
 
   while ((opt = getopt(argc, argv, "g:bcstw")) != -1) {
     switch (opt) {
@@ -897,9 +1014,9 @@ int md5sum_main(int argc,
       int fail;
       char *file = argv[optind];
 
-      fail = md5_file (file, binary, md5buffer);
+      fail = md5_file (file, md5buffer);
       err |= fail;
-      if (!fail && STREQ(file, "-")) {
+      if (!fail && file[0]=='-' && file[1] == '\0') {
 	  size_t i;
 	  for (i = 0; i < 16; ++i)
 	      printf ("%02x", md5buffer[i]);
