@@ -2,8 +2,8 @@
 /*
  * Mini syslogd implementation for busybox
  *
- * Copyright (C) 1999,2000,2001 by Lineo, inc.
- * Written by Erik Andersen <andersen@lineo.com>, <andersee@debian.org>
+ * Copyright (C) 1999,2000 by Lineo, inc. and Erik Andersen
+ * Copyright (C) 1999,2000,2001 by Erik Andersen <andersee@debian.org>
  *
  * Copyright (C) 2000 by Karl M. Hegbloom <karlheg@debian.org>
  *
@@ -55,7 +55,7 @@
 #define __LOG_FILE "/var/log/messages"
 
 /* Path to the unix socket */
-static char lfile[BUFSIZ] = "";
+static char lfile[BUFSIZ];
 
 static char *logFilePath = __LOG_FILE;
 
@@ -78,8 +78,15 @@ static int doRemoteLog = FALSE;
 static int local_logging = FALSE;
 #endif
 
+
+
 /* circular buffer variables/structures */
 #ifdef BB_FEATURE_IPC_SYSLOG
+#if __GNU_LIBRARY__ < 5
+#error Sorry.  Looks like you are using libc5.
+#error libc5 shm support isnt good enough.
+#error Please disable BB_FEATURE_IPC_SYSLOG
+#endif
 
 #include <sys/ipc.h>
 #include <sys/sem.h>
@@ -155,8 +162,6 @@ void ipcsyslog_init(void){
 		    perror_msg_and_die("semget");
 		}else
     			perror_msg_and_die("semget");
-	    } else {
-		    sem_up(s_semid);
 	    }
 	}else{
 		printf("Buffer already allocated just grab the semaphore?");
@@ -348,7 +353,7 @@ static const int IOV_COUNT = 2;
 		struct iovec iov[IOV_COUNT];
 		struct iovec *v = iov;
 
-		bzero(&res, sizeof(res));
+		memset(&res, 0, sizeof(res));
 		snprintf(res, sizeof(res), "<%d>", pri);
 		v->iov_base = res ;
 		v->iov_len = strlen(res);          
@@ -372,7 +377,7 @@ static const int IOV_COUNT = 2;
 
 static void quit_signal(int sig)
 {
-	logMessage(0, "System log daemon exiting.");
+	logMessage(LOG_SYSLOG | LOG_INFO, "System log daemon exiting.");
 	unlink(lfile);
 #ifdef BB_FEATURE_IPC_SYSLOG
 	ipcsyslog_cleanup();
@@ -389,21 +394,24 @@ static void domark(int sig)
 	}
 }
 
-static const int BUFSIZE = 1023;
+/* This must be a #define, since when DODEBUG and BUFFERS_GO_IN_BSS are
+ * enabled, we otherwise get a "storage size isn't constant error. */
+#define BUFSIZE 1023
 static int serveConnection (int conn)
 {
 	RESERVE_BB_BUFFER(tmpbuf, BUFSIZE + 1);
 	int    n_read;
+	char *p = tmpbuf;
 
 	n_read = read (conn, tmpbuf, BUFSIZE );
 
-	if (n_read > 0) {
+	while (p < tmpbuf + n_read) {
 
 		int           pri = (LOG_USER | LOG_NOTICE);
 		char          line[ BUFSIZE + 1 ];
 		unsigned char c;
 
-		char *p = tmpbuf, *q = line;
+		char *q = line;
 
 		tmpbuf[ n_read - 1 ] = '\0';
 
@@ -428,9 +436,11 @@ static int serveConnection (int conn)
 			p++;
 		}
 		*q = '\0';
+		p++;
 		/* Now log it */
 		logMessage (pri, line);
 	}
+	RELEASE_BB_BUFFER (tmpbuf);
 	return n_read;
 }
 
@@ -442,7 +452,7 @@ static void init_RemoteLog (void){
   struct hostent *hostinfo;
   int len = sizeof(remoteaddr);
 
-  bzero(&remoteaddr, len);
+  memset(&remoteaddr, 0, len);
 
   remotefd = socket(AF_INET, SOCK_DGRAM, 0);
 
@@ -450,11 +460,7 @@ static void init_RemoteLog (void){
     error_msg_and_die("syslogd: cannot create socket");
   }
 
-  hostinfo = (struct hostent *) gethostbyname(RemoteHost);
-
-  if (!hostinfo) {
-    error_msg_and_die("syslogd: cannot resolve remote host name [%s]", RemoteHost);
-  }
+  hostinfo = xgethostbyname(RemoteHost);
 
   remoteaddr.sin_family = AF_INET;
   remoteaddr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
@@ -494,17 +500,14 @@ static void doSyslogd (void)
 	alarm (MarkInterval);
 
 	/* Create the syslog file so realpath() can work. */
-	close (open (_PATH_LOG, O_RDWR | O_CREAT, 0644));
-	if (realpath (_PATH_LOG, lfile) == NULL)
-		perror_msg_and_die ("Could not resolve path to " _PATH_LOG);
-
-	unlink (lfile);
+	if (realpath (_PATH_LOG, lfile) != NULL)
+		unlink (lfile);
 
 	memset (&sunx, 0, sizeof (sunx));
 	sunx.sun_family = AF_UNIX;
 	strncpy (sunx.sun_path, lfile, sizeof (sunx.sun_path));
 	if ((sock_fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
-		perror_msg_and_die ("Couldn't obtain descriptor for socket " _PATH_LOG);
+		perror_msg_and_die ("Couldn't get file descriptor for socket " _PATH_LOG);
 
 	addrLength = sizeof (sunx.sun_family) + strlen (sunx.sun_path);
 	if ((bind (sock_fd, (struct sockaddr *) &sunx, addrLength)) || (listen (sock_fd, 5)))
@@ -528,7 +531,7 @@ static void doSyslogd (void)
         }
         #endif
 
-	logMessage (0, "syslogd started: BusyBox v" BB_VER " (" BB_BT ")");
+	logMessage (LOG_SYSLOG | LOG_INFO, "syslogd started: " BB_BANNER);
 
 	for (;;) {
 
@@ -587,11 +590,11 @@ extern int syslogd_main(int argc, char **argv)
 				doFork = FALSE;
 				break;
 			case 'O':
-				logFilePath = strdup(optarg);
+				logFilePath = xstrdup(optarg);
 				break;
 #ifdef BB_FEATURE_REMOTE_LOG
 			case 'R':
-				RemoteHost = strdup(optarg);
+				RemoteHost = xstrdup(optarg);
 				if ( (p = strchr(RemoteHost, ':'))){
 					RemotePort = atoi(p+1);
 					*p = '\0';
@@ -628,8 +631,12 @@ extern int syslogd_main(int argc, char **argv)
 	umask(0);
 
 	if (doFork == TRUE) {
+#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
 		if (daemon(0, 1) < 0)
 			perror_msg_and_die("daemon");
+#else
+			error_msg_and_die("daemon not supported");
+#endif
 	}
 	doSyslogd();
 
